DEFAULT_CONFIG = {
    "CLIENT_DEFAULT_DIMENSIONS": {},
    "CLIENT_TIMESTAMP_FORMAT": "%Y-%m-%dT%H:%M:%S.%fZ",
    "CLIENT_CHANNEL_TYPE" : "redis",
    "CLIENT_CHANNEL_ARGS" : None,
    "AGENT_LOGGER_NAME": "kadabra.agent",
    "AGENT_CHANNEL_TYPE" : "redis",
    "AGENT_CHANNEL_ARGS" : None,
    "AGENT_PUBLISHER_TYPE" : "debug",
    "AGENT_PUBLISHER_ARGS" : None,
    "AGENT_RECEIVER_THREADS" : 3,
    "AGENT_NANNY_FREQUENCY_SECONDS" : 30.0,
    "AGENT_NANNY_THRESHOLD_SECONDS" : 60.0,
    "AGENT_NANNY_QUERY_LIMIT": 5000,
    "AGENT_NANNY_THREADS": 3
}
